@@ -1,9 +1,9 @@
-;;; imenu-list.el --- Show imenu entries in a seperate buffer
+;;; imenu-list.el --- Show imenu entries in a separate buffer
 
-;; Copyright (C) 2015-2017 Bar Magal & Contributors
+;; Copyright (C) 2015-2021 Bar Magal & Contributors
 
 ;; Author: Bar Magal (2015)
-;; Version: 0.8
+;; Version: 0.9
 ;; Homepage: https://github.com/bmag/imenu-list
 ;; Package-Requires: ((cl-lib "0.5"))
 
@@ -316,6 +316,15 @@ buffer, or in other words: this hook is ran by both
         "Return t if X <= Y and Y <= Z."
         (and (<= x y) (<= y z)))))
 
+(defun imenu-list--translate-eglot-position (pos)
+  ;; when Eglot is in charge of Imenu, then the index is created by `eglot-imenu', with a fallback to
+  ;; `imenu-default-create-index-function' when `eglot-imenu' returns nil. If POS is an array, it means
+  ;; it was created by `eglot-imenu' and we need to extract its position. Otherwise, it was created by
+  ;; `imenu-default-create-index-function' and we should return it as-is.
+  (if (arrayp pos)
+      (eglot--lsp-position-to-point (plist-get (plist-get (aref pos 0) :range) :start) t)
+    pos))
+
 (defun imenu-list-position-translator ()
   "Get the correct position translator function for the current buffer.
 A position translator is a function that takes a position as described in
@@ -337,6 +346,8 @@ continue with the regular logic to find a translator function."
              (bound-and-true-p semantic-mode)))
     ;; semantic uses overlays, return overlay's start as position
     #'overlay-start)
+   ((and (fboundp #'eglot-managed-p) (eglot-managed-p))
+    #'imenu-list--translate-eglot-position)
    ;; default - return position as is
    (t #'identity)))
 
@@ -417,7 +428,11 @@ See `display-buffer-alist' for a description of BUFFER and ALIST."
   (or (get-buffer-window buffer)
       (let ((window (ignore-errors (split-window (frame-root-window) (imenu-list-split-size) imenu-list-position))))
         (when window
-          (window--display-buffer buffer window 'window alist t)
+          ;; since Emacs 27.0.50, `window--display-buffer' doesn't take a
+          ;; `dedicated' argument, so instead call `set-window-dedicated-p'
+          ;; directly (works both on new and old Emacs versions)
+          (window--display-buffer buffer window 'window alist)
+          (set-window-dedicated-p window t)
           window))))
 
 (defun imenu-list-install-display-buffer ()
@@ -637,7 +652,7 @@ ARG is ignored."
 
 ;;;###autoload
 (define-minor-mode imenu-list-minor-mode
-  nil :global t
+  nil :global t :group 'imenu-list
   (if imenu-list-minor-mode
       (progn
         (imenu-list-get-buffer-create)
